﻿using ProjectCleaningCar.Drawning;
using ProjectCleaningCar.MovementStrategy;

namespace ProjectCleaningCar;
/// <summary>
/// Форма работы с классом Подметально-уборочная машина
/// </summary>
public partial class FormCleaningCar : Form
{
    /// <summary>
    /// Поле-объект для прорисовки объекта
    /// </summary>
    private DrawningCar? _drawningCar;
    /// <summary>
    /// Стратегия перемещения
    /// </summary>
    private AbstractStrategy? _strategy;
    /// <summary>
    /// Конструктор формы
    /// </summary>
    public FormCleaningCar()
    {
        InitializeComponent();
        _strategy = null;
    }
    /// <summary>
    /// Метод прорисовки машины
    /// </summary>
    private void Draw()
    {
        if (_drawningCar == null)
        {
            return;
        }
        Bitmap bmp = new(pictureBoxCleaningCar.Width,
        pictureBoxCleaningCar.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawningCar.DrawTransport(gr);
        pictureBoxCleaningCar.Image = bmp;
    }
    /// <summary>
    /// Обработка нажатия кнопки "Создать подметательно-уборочную машину"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreateCleaningCar_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningCleaningCar));
    /// <summary>
    /// Обработка нажатия кнопки "Создать машину"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreateCar_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningCar));
    /// <summary>
    /// Создание объекта класса-перемещения
    /// </summary>
    /// <param name="type">Тип создаваемого объекта</param>
    public void CreateObject(string type)
    {
        Random random = new Random();
        switch (type)
        {
            case nameof(DrawningCar):
                _drawningCar = new DrawningCar(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 255), random.Next(0, 255), random.Next(0, 255)));
                break;
            case nameof(DrawningCleaningCar):
                _drawningCar = new DrawningCleaningCar(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 255), random.Next(0, 255), random.Next(0, 255)),
                    Color.FromArgb(random.Next(0, 255), random.Next(0, 255), random.Next(0, 255)),
                    Convert.ToBoolean(random.Next(0, 2)),
                    Convert.ToBoolean(random.Next(0, 2)),
                    Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }
        _drawningCar.SetPictureSize(pictureBoxCleaningCar.Width, pictureBoxCleaningCar.Height);
        _drawningCar.SetPosition(random.Next(0, 200), random.Next(0, 200));
        _strategy = null;
        comboBoxStrategy.Enabled = true;
        Draw();
    }
    /// <summary>
    /// Перемещение объекта по форме (нажатие кнопок навигации)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonMove_Click(object sender, EventArgs e)
    {
        if (_drawningCar == null)
        {
            return;
        }
        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawningCar.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawningCar.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawningCar.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawningCar.MoveTransport(DirectionType.Right);
                break;
        }
        if (result)
        {
            Draw();
        }
    }
    /// <summary>
    /// Обработка нажатия кнопки "Шаг"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawningCar == null)
        {
            return;
        }
        if (comboBoxStrategy.Enabled)
        {
            _strategy = comboBoxStrategy.SelectedIndex switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };
            if (_strategy == null)
            {
                return;
            }
            _strategy.SetData(new MoveableCar(_drawningCar),
            pictureBoxCleaningCar.Width, pictureBoxCleaningCar.Height);
        }
        if (_strategy == null)
        {
            return;
        }

        comboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();

        if (_strategy.GetStatus() == StrategyStatus.Finish)
        {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
        }
    }
}